package com.microsoft.office.ui.controls.inputpanel

import android.view.ViewTreeObserver

/**
 * An implementation of InputPanelStateController which does not runs
 * animation on input panel state updates. Here we attach View.OnGlobalLayoutListener
 * to receive a callback when the view is completely rendered on screen and only then
 * we intimate all the observers of InputPanel regarding the current state of InputPanel
 */
class AnimationRestrainedStateController(
        private val registry: IInputPanelRegistry,
        private val scanner: IInputStateScanner,
        inputPanelView: IInputPanelView
) : IInputPanelStateController, IInputPanelRegistry by registry {

    private val assertNotNull = assertNotNullCurried(inputPanelView)

    override fun updateState(state: Int, animate: Boolean) {
        val scannedState = scanner.scan(state)
        if (scannedState != null) {
            if (scannedState == InputPanelManager.PANEL_OPENED) {
                showInputPanel()
            } else if (scannedState == InputPanelManager.PANEL_CLOSED) {
                hideInputPanel()
            }
        }
    }

    private fun showInputPanel() {

        assertNotNull { view ->
            val layoutListener = object : ViewTreeObserver.OnGlobalLayoutListener {
                override fun onGlobalLayout() {
                    view.viewTreeObserver.removeOnGlobalLayoutListener(this)
                    notify { it.onInputPanelOpen() }
                }
            }

            view.viewTreeObserver.addOnGlobalLayoutListener(layoutListener)
            view.show()
        }
    }

    private fun hideInputPanel() {

        assertNotNull { view ->
            val layoutListener = object : ViewTreeObserver.OnGlobalLayoutListener {
                override fun onGlobalLayout() {
                    view.viewTreeObserver.removeOnGlobalLayoutListener(this)
                    notify { it.onInputPanelClosed() }
                }
            }

            view.viewTreeObserver.addOnGlobalLayoutListener(layoutListener)
            view.hide()
        }
    }

    private fun notify(block: (IInputPanelListener) -> Unit) {
        registry.listeners().forEach(block)
    }
}