package com.microsoft.office.ui.controls.inputpanel

import android.animation.Animator
import android.view.View
import android.view.ViewPropertyAnimator
import android.view.ViewTreeObserver
import android.view.animation.DecelerateInterpolator
import com.microsoft.office.ui.controls.inputpanel.InputPanelManager.PANEL_CLOSED
import com.microsoft.office.ui.controls.inputpanel.InputPanelManager.PANEL_OPENED

class InputPanelStateController(
        private val registry: IInputPanelRegistry,
        private val scanner: IInputStateScanner,
        inputPanelView: IInputPanelView
) : IInputPanelStateController, IInputPanelRegistry by registry {

    private var animator: ViewPropertyAnimator? = null
    private val assertNotNull = assertNotNullCurried(inputPanelView)

    override fun updateState(state: Int, animate: Boolean) {
        val scannedState = scanner.scan(state)
        if (scannedState != null) {
            if (scannedState == PANEL_OPENED) {
                showInputPanel(animate)
            } else if (scannedState == PANEL_CLOSED) {
                hideInputPanel(animate)
            }
        }
    }

    private fun showInputPanel(animate: Boolean) {
        assertNotNull { view ->
            if (animate) {
                showInputPanelWithAnimation(view)
            } else {
                showInputPanel(view)
            }
        }

    }

    private fun showInputPanelWithAnimation(view: View) {
        animateView(view,
                0F,
                animationStart = {
                    view.show()
                },
                animationEnd = {
                    notify { it.onInputPanelOpen() }
                }
        )
    }

    private fun showInputPanel(view: View) {
        val layoutListener = object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                view.viewTreeObserver.removeOnGlobalLayoutListener(this)
                notify { it.onInputPanelOpen() }
            }
        }

        view.viewTreeObserver.addOnGlobalLayoutListener(layoutListener)
        view.show()
    }

    private fun hideInputPanel(animate: Boolean) {
        assertNotNull { view ->
            if (animate) {
                hideInputPanelWithAnimation(view)
            } else {
                hideInputPanel(view)
            }
        }
    }

    private fun hideInputPanelWithAnimation(view: View) {
        animateView(view,
                view.height.toFloat(),
                animationEnd = {
                    view.hide()
                    notify { it.onInputPanelClosed() }
                },
                factor = {
                    1 - it
                }
        )
    }

    private fun hideInputPanel(view: View) {
        val layoutListener = object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                view.viewTreeObserver.removeOnGlobalLayoutListener(this)
                notify { it.onInputPanelClosed() }
            }
        }

        view.viewTreeObserver.addOnGlobalLayoutListener(layoutListener)
        view.hide()
    }

    private fun animateView(view: View,
                            to: Float,
                            animationStart: (Animator) -> Unit = { },
                            animationEnd: (Animator) -> Unit = { },
                            factor: (Float) -> Float = { it }) {

        animator?.cancel()
        animator = view.animate()
                .setDuration(DURATION)
                .translationY(to)
                .withLayer()
                .setInterpolator(DecelerateInterpolator())
                .setUpdateListener { animator ->
                    val animatedValue = animator.animatedValue as Float
                    notify { it.onInputPanelUpdate(factor(animatedValue)) }
                }
                .setAnimListener(
                        animationStart = {
                            animationStart(it)
                        },
                        animationEnd = {
                            animationEnd(it)
                        }
                )
    }

    private fun notify(block: (IInputPanelListener) -> Unit) {
        registry.listeners().forEach(block)
    }

    companion object {
        private const val DURATION = 300L
    }

}